<?php

namespace Drupal\openlayers\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;

use Drupal\openlayers\OpenlayersMapInterface;
use Drupal\openlayers\OpenlayersControlInterface;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

use Drupal\openlayers\MapControl;

/**
 * Provides a base form for ....
 */
abstract class MapControlFormBase extends FormBase {

  /**
   * The image style.
   *
   * @var \Drupal\image\ImageStyleInterface
   */
  protected $olMap;

  /**
   * The image effect.
   *
   * @var \Drupal\image\ImageEffectInterface|\Drupal\image\ConfigurableImageEffectInterface
   */
  protected $olControl;

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ol_map_control_form';
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\image\ImageStyleInterface $image_style
   *   The image style.
   * @param string $image_effect
   *   The image effect ID.
   *
   * @return array
   *   The form structure.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\NotFoundHttpException
   */
  public function buildForm(array $form, FormStateInterface $form_state, OpenlayersMapInterface $map = NULL, $control = NULL) {

    $request = $this->getRequest();
    $user_input = $form_state->getUserInput();
    $form['#attached']['library'][] = 'openlayers/admin';
    
    $this->olMap = $map;

    if (isset($this->olMap->controls[$control])) {
      $this->mapControl = new MapControl($this->olMap->controls[$control], 'update');
    } else {
      $this->mapControl = new MapControl($control, 'add');
    }

    $form['uuid'] = [
      '#type' => 'value',
      '#value' => $this->mapControl->getUuid(),
    ];
    $form['id'] = [
      '#type' => 'value',
      '#value' => $this->mapControl->id(),
    ];

    $form['data'] = [];
    $subform_state = SubformState::createForSubform($form['data'], $form, $form_state);
    $form['data'] = $this->mapControl->buildConfigurationForm($form['data'], $subform_state);  
    $form['data']['#tree'] = TRUE;

    // Check the URL for a weight, then the image effect, otherwise use default.
    $form['weight'] = [
      '#type' => 'hidden',
      '#value' => 0,
    ];

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#button_type' => 'primary',
      '#name' => 'add_control'
    ];
    $form['actions']['cancel'] = [
      '#type' => 'link',
      '#title' => $this->t('Cancel'),
      '#url' => $this->olMap->toUrl('edit-form'),
      '#attributes' => ['class' => ['button']],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // The image effect configuration is stored in the 'data' key in the form,
    // pass that through for validation.
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {    
    $type = str_replace('add_', '', $form_state->getTriggeringElement()['#name']);
    $form_state->cleanValues();

    // The image effect configuration is stored in the 'data' key in the form,
    // pass that through for submission.
    $this->{'map' . ucfirst($type)}->submitConfigurationForm($form['data'], SubformState::createForSubform($form['data'], $form, $form_state));
//    $this->{'map' . ucfirst($type)}->setWeight($form_state->getValue('weight'));

    if (!$this->{'map' . ucfirst($type)}->getUuid()) {

      $this->olMap->addMapObject($this->{'map' . ucfirst($type)}->getConfiguration(), $type);     //  TODO - where is the addMapLayer function (answer: OpenlayersMap.php) ?
    } else {

      $this->olMap->updateMapObject($this->{'map' . ucfirst($type)}->getConfiguration(), $type);
    }

    $this->olMap->save();
    
    $this->messenger()->addStatus($this->t('The ' . $type . ' was successfully applied to the map.'));
    $form_state->setRedirectUrl($this->olMap->toUrl('edit-form'));
  }

  /**
   * Converts an image effect ID into an object.
   *
   * @param string $image_effect
   *   The image effect ID.
   *
   * @return \Drupal\image\ImageEffectInterface
   *   The image effect object.
   */
  abstract protected function prepareOpenlayersControl($control);

}
