<?php

namespace Drupal\openlayers\Plugin\Control\ScaleLine;

use Drupal\Core\Form\FormStateInterface;
use Drupal\openlayers\Types\Control;

//use Drupal\Component\Utility\Color;
//use Drupal\Component\Utility\Rectangle;


/**
 * Defines the Scale Line control for an Openlayers map.
 *
 * @OpenlayersPlugin(
 *   id = "ol_control_scaleline",
 *   label = @Translation("Scale Line"),
 *   description = @Translation("Define a scale line control."),
 *   service = "openlayers.Control:ScaleLine",
 *   library = "openlayers-plugin-control-scaleline",
 *   is_configurable = "false",
 *   type = "control"
 * )
 */
class ScaleLine extends Control {

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $summary = [
      '#theme' => 'openlayers_control_summary',            //  TODO - ???
      '#data' => $this->configuration,
    ];
    $summary += parent::getSummary();

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'className' => 'ol-scale-line',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['className'] = [
      '#type' => 'textfield',
      '#default_value' => $this->configuration['className'],
      '#title' => t('Class name'),
      '#description' => t('CSS class name.'),
      '#size' => 50,
      '#maxlength' => 50,
      '#required' => FALSE,
    ];
    $form['collapsible'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Collapsible'),
      '#description' => t('Specify if attributions can be collapsed.'),
//      '#default_value' => $this->configuration['collapsible'],
    );
    $form['collapsed'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Collapsed'),
      '#description' => t('Specify if attributions should be collapsed at startup.'),
//      '#default_value' => $this->configuration['collapsed'],        //  Specify if attributions can be collapsed.
    );
    $form['tipLabel'] = [
      '#type' => 'textfield',
      '#title' => t('Tip label'),
      '#description' => t('Text label to use for the button tip.'),
//      '#default_value' => $this->configuration['tipLabel'],
      '#size' => 100,
      '#maxlength' => 100,
      '#required' => FALSE,
    ];
    $form['label'] = [
      '#type' => 'textfield',
//      '#default_value' => $this->configuration['label'],
      '#title' => t('Label'),
      '#description' => t('Text label to use for the collapsed attributions button.'),
      '#size' => 100,
      '#maxlength' => 100,
      '#required' => FALSE,
    ];
    $form['collapseLabel'] = [
      '#type' => 'textfield',
//      '#default_value' => $this->configuration['collapseLabel'],
      '#title' => t('Collapse label'),
      '#description' => t('Text label to use for the expanded attributions button.'),
      '#size' => 100,
      '#maxlength' => 100,
      '#required' => FALSE,
    ];
    
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
/*
    if (!$form_state->isValueEmpty('color') && !Color::validateHex($form_state->getValue('color'))) {
      $form_state->setErrorByName('color', $this->t('Color must be a hexadecimal color value.'));
    }
*/
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $this->configuration['className'] = $form_state->getValue('className');
/*
    $this->configuration['collapsible'] = $form_state->getValue('collapsible');
    $this->configuration['collapsed'] = $form_state->getValue('collapsed');
    $this->configuration['tipLabel'] = $form_state->getValue('tipLabel');
    $this->configuration['label'] = $form_state->getValue('label');
    $this->configuration['collapseLabel'] = $form_state->getValue('collapseLabel');
*/
  }

}
