<?php

namespace Drupal\openlayers\Plugin\Control\ZoomToExtent;

use Drupal\Core\Form\FormStateInterface;
use Drupal\openlayers\Types\Control;

/**
 * Defines the Zoom To Extent control for an Openlayers map.
 *
 * @OpenlayersPlugin(
 *   id = "ol_control_zoomtoextent",
 *   label = @Translation("Zoom to Extent"),
 *   description = @Translation("Define a zoom to extent control."),
 *   service = "openlayers.Control:ZoomToExtent",
 *   library = "openlayers-plugin-control-zoomtoextent",
 *   is_configurable = "false",
 *   type = "control"
 * )
 */
class ZoomToExtent extends Control {

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $summary = [
      '#theme' => 'openlayers_control_summary',
      '#data' => $this->configuration,
    ];
    $summary += parent::getSummary();

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'className' => 'ol-zoomtoextent',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['className'] = [
      '#type' => 'textfield',
      '#default_value' => $this->configuration['className'],
      '#title' => t('Class name'),
      '#description' => t('CSS class name.'),
      '#size' => 50,
      '#maxlength' => 50,
      '#required' => FALSE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $this->configuration['className'] = $form_state->getValue('className');
  }

}
